@@ -1,32 +1,5 @@
-///////////////////////////////////////////////////////////////////////////
-//
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-//
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
-//
-// Redistributions of source code must retain the above copyright
-// and license notice and the following restrictions and disclaimer.
-//
-// *     Neither the name of DreamWorks Animation nor the names of
-// its contributors may be used to endorse or promote products derived
-// from this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-// IN NO EVENT SHALL THE COPYRIGHT HOLDERS' AND CONTRIBUTORS' AGGREGATE
-// LIABILITY FOR ALL CLAIMS REGARDLESS OF THEIR BASIS EXCEED US$250.00.
-//
-///////////////////////////////////////////////////////////////////////////
+// Copyright Contributors to the OpenVDB Project
+// SPDX-License-Identifier: MPL-2.0
 
 /// @file Clip.h
 ///
@@ -40,6 +13,7 @@
 #include <openvdb/math/Math.h> // for math::isNegative()
 #include <openvdb/math/Maps.h> // for math::NonlinearFrustumMap
 #include <openvdb/tree/LeafManager.h>
+#include <openvdb/points/PointDataGrid.h>
 #include "GridTransformer.h" // for tools::resampleToMatch()
 #include "Prune.h"
 #include <tbb/blocked_range.h>
@@ -62,7 +36,7 @@ namespace tools {
 /// @warning Clipping a level set will likely produce a grid that is
 /// no longer a valid level set.
 template<typename GridType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 clip(const GridType& grid, const BBoxd& bbox, bool keepInterior = true);
 
 /// @brief Clip the given grid against a frustum and return a new grid containing the result.
@@ -73,7 +47,7 @@ clip(const GridType& grid, const BBoxd& bbox, bool keepInterior = true);
 /// @warning Clipping a level set will likely produce a grid that is
 /// no longer a valid level set.
 template<typename GridType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 clip(const GridType& grid, const math::NonlinearFrustumMap& frustum, bool keepInterior = true);
 
 /// @brief Clip a grid against the active voxels of another grid
@@ -89,12 +63,13 @@ clip(const GridType& grid, const math::NonlinearFrustumMap& frustum, bool keepIn
 /// @warning Clipping a level set will likely produce a grid that is
 /// no longer a valid level set.
 template<typename GridType, typename MaskTreeType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 clip(const GridType& grid, const Grid<MaskTreeType>& mask, bool keepInterior = true);
 
 
 ////////////////////////////////////////
 
+/// @cond OPENVDB_DOCS_INTERNAL
 
 namespace clip_internal {
 
@@ -267,7 +242,7 @@ struct ConvertGrid<GridT, GridT>
 // and return a pointer to the new grid.
 /// @private
 template<typename GridT>
-inline typename std::enable_if<!std::is_same<MaskValueType, typename GridT::BuildType>::value,
+typename std::enable_if<!std::is_same<MaskValueType, typename GridT::BuildType>::value,
     typename GridT::template ValueConverter<MaskValueType>::Type::Ptr>::type
 convertToMaskGrid(const GridT& grid)
 {
@@ -281,7 +256,7 @@ convertToMaskGrid(const GridT& grid)
 // Overload that avoids any processing if the input grid is already a mask grid
 /// @private
 template<typename GridT>
-inline typename std::enable_if<std::is_same<MaskValueType, typename GridT::BuildType>::value,
+typename std::enable_if<std::is_same<MaskValueType, typename GridT::BuildType>::value,
     typename GridT::ConstPtr>::type
 convertToMaskGrid(const GridT& grid)
 {
@@ -294,7 +269,7 @@ convertToMaskGrid(const GridT& grid)
 
 /// @private
 template<typename GridType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 doClip(
     const GridType& grid,
     const typename GridType::template ValueConverter<MaskValueType>::Type& clipMask,
@@ -329,11 +304,7 @@ doClip(
         gridMask.topologyDifference(clipMask.constTree());
     }
 
-#if OPENVDB_ABI_VERSION_NUMBER <= 3
-    auto outGrid = grid.copy(CP_NEW);
-#else
     auto outGrid = grid.copyWithNewTree();
-#endif
     {
         // Copy voxel values and states.
         tree::LeafManager<const MaskTreeT> leafNodes(gridMask);
@@ -369,13 +340,15 @@ doClip(
 
 } // namespace clip_internal
 
+/// @endcond
+
 
 ////////////////////////////////////////
 
 
 /// @private
 template<typename GridType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 clip(const GridType& grid, const BBoxd& bbox, bool keepInterior)
 {
     using MaskValueT = clip_internal::MaskValueType;
@@ -396,7 +369,7 @@ clip(const GridType& grid, const BBoxd& bbox, bool keepInterior)
 
 /// @private
 template<typename SrcGridType, typename ClipTreeType>
-inline typename SrcGridType::Ptr
+typename SrcGridType::Ptr
 clip(const SrcGridType& srcGrid, const Grid<ClipTreeType>& clipGrid, bool keepInterior)
 {
     using MaskValueT = clip_internal::MaskValueType;
@@ -427,7 +400,7 @@ clip(const SrcGridType& srcGrid, const Grid<ClipTreeType>& clipGrid, bool keepIn
 
 /// @private
 template<typename GridType>
-inline typename GridType::Ptr
+typename GridType::Ptr
 clip(const GridType& inGrid, const math::NonlinearFrustumMap& frustumMap, bool keepInterior)
 {
     using ValueT = typename GridType::ValueType;
@@ -462,11 +435,7 @@ clip(const GridType& inGrid, const math::NonlinearFrustumMap& frustumMap, bool k
     };
 
     // Construct an output grid with the same transform and metadata as the input grid.
-#if OPENVDB_ABI_VERSION_NUMBER <= 3
-    auto outGrid = inGrid.copy(CP_NEW);
-#else
     auto outGrid = inGrid.copyWithNewTree();
-#endif
     if (outGrid->getGridClass() == GRID_LEVEL_SET) {
         // After clipping, a level set grid might no longer be a valid SDF.
         outGrid->setGridClass(GRID_UNKNOWN);
@@ -595,12 +564,38 @@ clip(const GridType& inGrid, const math::NonlinearFrustumMap& frustumMap, bool k
     return outGrid;
 }
 
+
+////////////////////////////////////////
+
+
+// Explicit Template Instantiation
+
+#ifdef OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+#ifdef OPENVDB_INSTANTIATE_CLIP
+#include <openvdb/util/ExplicitInstantiation.h>
+#endif
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr clip(const Grid<TreeT>&, const BBoxd&, bool)
+OPENVDB_VOLUME_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr clip(const Grid<TreeT>&, const math::NonlinearFrustumMap&, bool)
+OPENVDB_ALL_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#define _FUNCTION(TreeT) \
+    Grid<TreeT>::Ptr clip_internal::doClip(const Grid<TreeT>&, const MaskGrid&, bool)
+OPENVDB_VOLUME_TREE_INSTANTIATE(_FUNCTION)
+#undef _FUNCTION
+
+#endif // OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
 
 #endif // OPENVDB_TOOLS_CLIP_HAS_BEEN_INCLUDED
-
-// Copyright (c) 2012-2019 DreamWorks Animation LLC
-// All rights reserved. This software is distributed under the
-// Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
